#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['qmk_cli']

package_data = \
{'': ['*']}

install_requires = \
['argcomplete', 'colorama']

entry_points = \
{'console_scripts': ['qmk = qmk_cli.script_qmk:main']}

setup(name='qmk',
      version='0.0.1',
      description='A program to help you work with qmk_firmware.',
      author='skullydazed',
      author_email='skullydazed@gmail.com',
      url='https://github.com/qmk/qmk_cli',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.5',
     )
